
#include <avr/io.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdbool.h>
#include "I2C.h"
#include "lcd.h"

//adresa device-ului din DataSheet
#define Device_Write_address	0xD0	
//setare bit LSB de la adresa slave pentru citire                       
#define Device_Read_address		0xD1							
int second, minute, hour, day, date, month, year;

//functie care incrementeaza un numar care e scris ca sir de caractere
char *
incr(char *s) 
{
	int i, begin, tail, len;
	int neg = (*s == '-');
	char tgt = neg ? '0' : '9';
	len = strlen(s);
	begin = (*s == '-' || *s == '+') ? 1 : 0;
	for (tail = len - 1; tail >= begin && s[tail] == tgt; tail--) ;
		if (tail < begin && !neg)
		{
			for (i = 1; i <= len - begin; i++)
				s[i] = '0';
			s[len + 1] = '\0';
		}
		else if (tail == begin && neg && s[1] == '1')
		{
			for (i = 1; i < len - begin; i++)
				s[i] = '9';
			s[len - 1] = '\0';
		}
		else
		{
			for (i = len - 1; i > tail; i--)
				s[i] = neg ? '9' : '0';
			s[tail] += neg ? -1 : 1;
		}
		return s;
	}
//functie care verifica daca este AM or PM
int IsItPM(char hour_){
	if (hour_ & (0x20))
		return 1;
	else
		return 0;
}

/*
functie care citeste de pe RTC
Se face comunicatia prin I2C cu RTC
si se incepe a se citi de la adresa data(0
pentru clock si 3 pentru calendar)
*/ 
void RTC_Read_Clock(char read_clock_address) {
		I2C_Start(Device_Write_address);
		I2C_Write(read_clock_address);
		I2C_Repeated_Start(Device_Read_address);
		second = I2C_Read_Ack();
		minute = I2C_Read_Ack();
		hour = I2C_Read_Nack();
		I2C_Stop();
} 

/*
asemanator ca pentru clock
*/ 
void RTC_Read_Calendar(char read_calendar_address) {
		I2C_Start(Device_Write_address);
		I2C_Write(read_calendar_address);
		I2C_Repeated_Start(Device_Read_address);
		day = I2C_Read_Ack();
		date = I2C_Read_Ack();
		month = I2C_Read_Ack();
		year = I2C_Read_Nack();
		I2C_Stop();
} 

//BUTON PD6 modificare ora alarma
//BUTON PD4 modificare minute alarma
//PORTB setare alarma inactiva 0x00
//setare alarma activa la activare alarmei 0xff
int main(void) {
		char buffer[20];
		char *days[7];
		days[0] = (char*) "Sun";
		days[1] = (char*) "Mon";
		days[2] = (char*) "Tue";
		days[3] = (char*) "Wed";
		days[4] = (char*) "Thu";
		days[5] = (char*) "Fri";
		days[6] = (char*) "Sat";
	    //initializare I2C
		I2C_Init();
		DDRB = 0xff;
		DDRD &= ~(1 << PD6) & ~(1 << PD4);
		PORTD |= (1 << PD6) & ~(1 << PD4);
		PORTB = 0x00;
		//initializare LCD                                                              
		LCD_init();
		char *ret_1 = (char *)malloc(sizeof(char) * 3);
		strcpy(ret_1, "00\0");
		char *ret_2 = (char *)malloc(sizeof(char) * 3);
		strcpy(ret_2, "00\0");
		int check = 0;
		int set = 0;
		while (1) {
		    //citire de la adresa 0 RTC
			RTC_Read_Clock(0);
		    //daca butonul de la PD6 este apasat atunci se doreste introducerea
		    //unei alarme. Dupa prin tinerea apasata pe butonul PD6 se va modifica
		    //ora
			if (check == 0 && (PIND & (1 << PD6))) {
				_delay_ms(20);
				check = 1;
				LCD_writeInstr(LCD_INSTR_clearDisplay);
				LCD_printAt(0, "Enter Alarm");
				sprintf(buffer, "%s::%s", ret_1, ret_2);
				LCD_printAt(40, buffer);
			}
			if (check == 1 && (PIND & (1 << PD6))){
				_delay_ms(20);
				ret_1 = incr(ret_1);
				if (strcmp(ret_1, "013") == 0
					|| strcmp(ret_1, "13") == 0)

				{
					strcpy(ret_1, "01");
				}
				sprintf(buffer, "%s:%s", ret_1, ret_2);
				LCD_printAt(40, buffer);
				PORTB = 0xff;
				_delay_ms(600);
			}
		    //daca butonul de la PD4 e apasat atunci se va modifica minutele
		    //la care se doreste alarma
			if (check == 1 && (PIND & (1 << PD4)))
			{
				_delay_ms(20);
				set = 1;
				ret_2 = incr(ret_2);
				if (strcmp(ret_2, "60") == 0)
				{
					strcpy(ret_2, "00");
				}
				sprintf(buffer, "%s:%s", ret_1, ret_2);
				LCD_printAt(40, buffer);
				_delay_ms(600);
			}
			//atunci cand butonul de la PD4 alarma va fi setata
			//si se va reveni la a afisa ceasul
			if (check == 1 && !(PIND & (1 << PD4)) && set == 1)
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);
				check = 0;
				set = 0;
				LCD_printAt(0,"AlarmaSetata");
				_delay_ms(6000);
			}
			//afisare ceas in mod normal
			if (check == 0)
			{
				sprintf(buffer, "%02x:%02x:%02x  ",
					(hour & 0b00011111), 
					minute, second);
				if (IsItPM(hour))
					strcat(buffer, "PM");
				else
					strcat(buffer, "AM");
				LCD_printAt(0, buffer);
				RTC_Read_Calendar(3);
				sprintf(buffer, "%02x/%02x/%02x %3s ", date, month,
					year, 
					days[day - 1]);
				LCD_printAt(40, buffer);
			}
		}
	}
